<?php

namespace Drupal\jira_issue_collector\Access;

use \Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountProxyInterface;

/**
 * Determine Jira Issue Collector visibility based on path/roles.
 *
 * @ingroup jira_access
 */
class AccessManager implements AccessManagerInterface {

  /**
   * The alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * The path matcher under test.
   *
   * @var \Drupal\Core\Path\PathMatcher
   */
  protected $pathMatcher;

  /**
   * The jira_issue_collector.settings config object.
   *
   * @var \Drupal\Core\Config\Config
   */
  protected $config;

  /**
   * The current path stack.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected $currentPath;

  /**
   * The current user object.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * Constructs the AccessManager.
   *
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   The path matcher.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path stack.
   * @param \Drupal\Core\Session\AccountProxyInterface $current_user
   *   The current user object.
   */
  public function __construct(AliasManagerInterface $alias_manager, PathMatcherInterface $path_matcher, ConfigFactoryInterface $config_factory, CurrentPathStack $current_path, AccountProxyInterface $current_user) {
    $this->aliasManager = $alias_manager;
    $this->pathMatcher = $path_matcher;
    $this->config = $config_factory->get('jira_issue_collector.settings');
    $this->currentPath = $current_path;
    $this->currentUser = $current_user;
  }

  /**
   * {@inheritdoc}
   */
  public function visibilityRoles() {
    $enabled = FALSE;
    // Filter the array to remove any non-selected roles. They will appear as
    // entries with value set to 0.
    $roles = array_filter($this->config->get('roles'));
    $current_user_roles = $this->currentUser->getAccount()->getRoles();

    if (!empty($roles)) {
      // One or more roles are selected for tracking.
      foreach (array_values($current_user_roles) as $rid) {
        // Is the current user a member of one role enabled for tracking?
        if (isset($roles[$rid]) && $rid == $roles[$rid]) {
          // Current user is a member of a role that should be shown the
          // JIRA Issue Collector widget.
          $enabled = TRUE;
          break;
        }
      }
    }
    else {
      // No role is selected for tracking, therefor the widget should be shown
      // for all roles.
      $enabled = TRUE;
    }

    return $enabled;
  }

  /**
   * {@inheritdoc}
   */
  public function visibilityPages() {
    $visibility = $this->config->get('visibility');
    $pages = $this->config->get('pages');

    // Match path if necessary.
    if (!empty($pages)) {
      if ($visibility < 2) {
        $current_path = $this->currentPath->getPath();
        $path = $this->aliasManager->getAliasByPath($current_path);

        // Compare with the internal and path alias (if any).
        $page_match = $this->pathMatcher->matchPath($path, $pages);
        if ($path != $current_path) {
          $page_match = $page_match || $this->pathMatcher->matchPath($current_path, $pages);
        }
        // When $visibility has a value of 0, the block is displayed on
        // all pages except those listed in $pages. When set to 1, it
        // is displayed only on those pages listed in $pages.
        $page_match = !($visibility xor $page_match);
      }
      else {
        $page_match = FALSE;
      }
    }
    else {
      $page_match = TRUE;
    }

    return $page_match;
  }

}
